--[[

Creates the main KillTracker master log viewer window, and provides a function
to refresh the table when the data is updated.

]]--

-- create and initialize the main KillTracker UI window
function _G.CreateMainWindow()
   -- show all characters' data or only the current character's?
   _G.showAllCharacters = settings.defaultAllChars;
   
	-- lock the quick open icon's position?
   _G.iconPosLocked = settings.iconLocked;

	if (settings.defaultSort == "KillTime") then
	   _G.sortByKillTime = true;
		_G.currentSort = "Enemy";	
		_G.currentSortOrder = "DESC";
	else
		_G.sortByKillTime = false;	
		_G.currentSort = settings.defaultSort;	
		_G.currentSortOrder = settings.defaultSortOrd;
	end
   
   _G.noDataDisplayed = true;

   -- the main window
	_G.statsWindow = ResizableWindow();

   statsWindow.minimumWidth = minWidth;
	statsWindow.minimumHeight = minHeight;

   statsWindow:SetSize(settings.width, settings.height);
	statsWindow:SetPosition(settings.leftPos, settings.topPos);
	statsWindow:SetText("KillTracker");
	statsWindow:SetZOrder(1);

   -- the main container for the kill log entries
   statsWindow.statsTable = Turbine.UI.ListBox();
   statsWindow.statsTable:SetParent(statsWindow.content);
	statsWindow.statsTable:SetPosition(15, 45);
	statsWindow.statsTable:SetSize(statsWindow.content:GetWidth() - 35, statsWindow.content:GetHeight() - 75);

   -- vertical scrollbar
   statsWindow.vscroll = Turbine.UI.Lotro.ScrollBar();
   statsWindow.vscroll:SetParent(statsWindow.content);
   statsWindow.vscroll:SetOrientation(Turbine.UI.Orientation.Vertical);
   statsWindow.vscroll:SetPosition(statsWindow.statsTable:GetLeft() + statsWindow.statsTable:GetWidth(), statsWindow.statsTable:GetTop());
   statsWindow.vscroll:SetSize(12, statsWindow.statsTable:GetHeight());
   statsWindow.statsTable:SetVerticalScrollBar(statsWindow.vscroll);

	-- add toggle checkbox to display all characters' data
   statsWindow.allCharactersToggle = Turbine.UI.Lotro.CheckBox();
	statsWindow.allCharactersToggle:SetParent(statsWindow.content);
	statsWindow.allCharactersToggle:SetSize((statsWindow:GetWidth() / 2) - 40, 20);
	statsWindow.allCharactersToggle:SetPosition(40, statsWindow:GetHeight() - 45);
	statsWindow.allCharactersToggle:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	statsWindow.allCharactersToggle:SetChecked(showAllCharacters);
   statsWindow.allCharactersToggle:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
   statsWindow.allCharactersToggle:SetForeColor(Turbine.UI.Color(1,.9,.5));
   statsWindow.allCharactersToggle:SetOutlineColor(Turbine.UI.Color(0,0,0));
   statsWindow.allCharactersToggle:SetFontStyle(Turbine.UI.FontStyle.Outline);
	statsWindow.allCharactersToggle:SetMultiline(false);
	statsWindow.allCharactersToggle:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	statsWindow.allCharactersToggle:SetText(" "..L.ShowAllCharacters);
	statsWindow.allCharactersToggle.Icon = Turbine.UI.Control();
	statsWindow.allCharactersToggle.Icon:SetParent(statsWindow.allCharactersToggle);
	statsWindow.allCharactersToggle.Icon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	statsWindow.allCharactersToggle.Icon:SetSize(32, 16);
	statsWindow.allCharactersToggle.Icon:SetPosition(0, 2);
	statsWindow.allCharactersToggle.Icon:SetMouseVisible(false);
	statsWindow.allCharactersToggle.CheckedChanged = function(sender, args)
		if sender:IsChecked() then
			_G.showAllCharacters = true;

			statsWindow.noRowsMessage:SetText(L.NoRowsAllChars);
		else
			_G.showAllCharacters = false;

			statsWindow.noRowsMessage:SetText(L.NoRowsCurrentChar);
         
			-- move the sort to the first column if hiding the sorted column
         if (currentSort == "Character") then
         	_G.currentSort = "Enemy";
				_G.currentSortOrder = "DESC";	

         	if (not sortByKillTime) then
         		statsWindow.headerRow[1]:SetText(L.MobName.." "..DescSortChar);
         	end

				statsWindow.headerRow[2]:SetText(L.CharacterName);
         end
		end

		RefreshKillsDisplay();
		options:SetCurrentSortText();

      statsWindow.content.Layout();
	end

	-- add toggle checkbox to swith to/from "Sort By Kill Time"
   statsWindow.killTimeToggle = Turbine.UI.Lotro.CheckBox();
	statsWindow.killTimeToggle:SetParent(statsWindow.content);
	statsWindow.killTimeToggle:SetSize((statsWindow:GetWidth() / 2) - 40, 20);
	statsWindow.killTimeToggle:SetPosition((statsWindow:GetWidth() / 2), statsWindow:GetHeight() - 45);
	statsWindow.killTimeToggle:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	if (sortByKillTime) then
		statsWindow.killTimeToggle:SetChecked(true);
	else
		statsWindow.killTimeToggle:SetChecked(false);
	end
   statsWindow.killTimeToggle:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
   statsWindow.killTimeToggle:SetForeColor(Turbine.UI.Color(1,.9,.5));
   statsWindow.killTimeToggle:SetOutlineColor(Turbine.UI.Color(0,0,0));
   statsWindow.killTimeToggle:SetFontStyle(Turbine.UI.FontStyle.Outline);
	statsWindow.killTimeToggle:SetMultiline(false);
	statsWindow.killTimeToggle:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	statsWindow.killTimeToggle:SetText(" "..L.SortByKillTime);
	statsWindow.killTimeToggle.Icon = Turbine.UI.Control();
	statsWindow.killTimeToggle.Icon:SetParent(statsWindow.killTimeToggle);
	statsWindow.killTimeToggle.Icon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	statsWindow.killTimeToggle.Icon:SetSize(32, 16);
	statsWindow.killTimeToggle.Icon:SetPosition(0, 2);
	statsWindow.killTimeToggle.Icon:SetMouseVisible(false);
	statsWindow.killTimeToggle.CheckedChanged = function(sender, args)
		if (sender:IsChecked()) then
         _G.sortByKillTime = true;
         
			statsWindow.headerRow[1]:SetText(L.MobName);
			statsWindow.headerRow[2]:SetText(L.CharacterName);
			statsWindow.headerRow[3]:SetText(L.LevelRange);
			statsWindow.headerRow[4]:SetText(L.KillCount);
		else
         _G.sortByKillTime = false;

			if (currentSort == "Enemy") then
				if (currentSortOrder == "ASC") then
					statsWindow.headerRow[1]:SetText(L.MobName.." "..AscSortChar);
				else
					statsWindow.headerRow[1]:SetText(L.MobName.." "..DescSortChar);
				end
			else
				statsWindow.headerRow[1]:SetText(L.MobName);
			end

			if (currentSort == "Character") then
				if (currentSortOrder == "ASC") then
					statsWindow.headerRow[2]:SetText(L.CharacterName.." "..AscSortChar);
				else
					statsWindow.headerRow[2]:SetText(L.CharacterName.." "..DescSortChar);
				end
			else
				statsWindow.headerRow[2]:SetText(L.CharacterName);
			end

			if (currentSort == "Level") then
				if (currentSortOrder == "ASC") then
					statsWindow.headerRow[3]:SetText(L.LevelRange.." "..AscSortChar);
				else
					statsWindow.headerRow[3]:SetText(L.LevelRange.." "..DescSortChar);
				end
			else
				statsWindow.headerRow[3]:SetText(L.LevelRange);
			end

			if (currentSort == "Kills") then
				if (currentSortOrder == "ASC") then
					statsWindow.headerRow[4]:SetText(L.KillCount.." "..AscSortChar);
				else
					statsWindow.headerRow[4]:SetText(L.KillCount.." "..DescSortChar);
				end
			else
				statsWindow.headerRow[4]:SetText(L.KillCount);
			end
		end

		RefreshKillsDisplay();
		options:SetCurrentSortText();
	end

	-- add "Save Disabled" message for data file load failures
   statsWindow.dataFailure = Turbine.UI.Label();
   statsWindow.dataFailure:SetParent(statsWindow.content);
   statsWindow.dataFailure:SetSize(statsWindow.content:GetWidth() - 80, 12);
   statsWindow.dataFailure:SetPosition(40, 15);
   statsWindow.dataFailure:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter); 
   statsWindow.dataFailure:SetFont(Turbine.UI.Lotro.Font.Arial12);
   statsWindow.dataFailure:SetForeColor(Turbine.UI.Color(1,0,0));
	statsWindow.dataFailure:SetMultiline(false);
   statsWindow.dataFailure:SetText(L.SaveDisabled);
   statsWindow.dataFailure:SetVisible(false);

	-- add Options link
   statsWindow.optionsLink = Turbine.UI.Label();
   statsWindow.optionsLink:SetParent(statsWindow.content);
   statsWindow.optionsLink:SetSize(80, 12);
   statsWindow.optionsLink:SetPosition(statsWindow.content:GetWidth() - 115, 7);
   statsWindow.optionsLink:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight); 
   statsWindow.optionsLink:SetFont(Turbine.UI.Lotro.Font.Verdana10);
   statsWindow.optionsLink:SetForeColor(Turbine.UI.Color(0,1,1));
	statsWindow.optionsLink:SetMultiline(false);
   statsWindow.optionsLink:SetText(L.OptionsLink);
	statsWindow.optionsLink.MouseClick = function(sender, args)
		Turbine.PluginManager.ShowOptions(Plugins.KillTracker);
	end

   local tblWidth = statsWindow.statsTable:GetWidth();
      
   -- add column headers
   statsWindow.headerRow = {};

   local tmpRow = Turbine.UI.Control();
   tmpRow:SetParent(statsWindow.content);
   tmpRow:SetPosition(15, 25);
   tmpRow:SetSize(tblWidth, 20);
         
   -- target name header
   statsWindow.headerRow[1] = Turbine.UI.Label();
   statsWindow.headerRow[1]:SetParent(tmpRow);
   statsWindow.headerRow[1]:SetSize((tblWidth - 80 - 100 - 10), 20);
   statsWindow.headerRow[1]:SetPosition(5, 0);
   statsWindow.headerRow[1]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft); 
   statsWindow.headerRow[1]:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
   statsWindow.headerRow[1]:SetForeColor(Turbine.UI.Color(1,.9,.5));
   statsWindow.headerRow[1]:SetOutlineColor(Turbine.UI.Color(0,0,0));
   statsWindow.headerRow[1]:SetFontStyle(Turbine.UI.FontStyle.Outline);
	statsWindow.headerRow[1]:SetMultiline(false);
	if ((currentSort == "Enemy") and (not sortByKillTime)) then
		if (currentSortOrder == "ASC") then
			statsWindow.headerRow[1]:SetText(L.MobName.." "..AscSortChar);
		else
			statsWindow.headerRow[1]:SetText(L.MobName.." "..DescSortChar);
		end
	else
		statsWindow.headerRow[1]:SetText(L.MobName);
	end
	statsWindow.headerRow[1].MouseClick = function(sender, args)
		if (not noDataDisplayed) then
			if (currentSort == "Enemy") then
				if (((currentSortOrder == "ASC") and (not sortByKillTime))
						or ((currentSortOrder == "DESC") and (sortByKillTime))) then
					_G.currentSortOrder = "DESC";
					statsWindow.headerRow[1]:SetText(L.MobName.." "..DescSortChar);
				else
					_G.currentSortOrder = "ASC";
					statsWindow.headerRow[1]:SetText(L.MobName.." "..AscSortChar);
				end
			else
				_G.currentSort = "Enemy";
				_G.currentSortOrder = "DESC"; 
	
				statsWindow.headerRow[1]:SetText(L.MobName.." "..DescSortChar);
				statsWindow.headerRow[2]:SetText(L.CharacterName);
				statsWindow.headerRow[3]:SetText(L.LevelRange);
				statsWindow.headerRow[4]:SetText(L.KillCount);
			end
					
	      RefreshKillsDisplay();
			options:SetCurrentSortText();
	   end

		_G.sortByKillTime = false;
		statsWindow.killTimeToggle:SetChecked(false);
	end
   -- character name(server ID) header
   statsWindow.headerRow[2] = Turbine.UI.Label();
   statsWindow.headerRow[2]:SetParent(tmpRow);
	if (showAllCharacters) then
   	statsWindow.headerRow[2]:SetSize(((tblWidth - 80 - 100 - 10) * 0.3) - 5 + 20, 20);
	else
   	statsWindow.headerRow[2]:SetSize(0, 20);
   end
   statsWindow.headerRow[2]:SetPosition(((tblWidth - 80 - 100 - 10) * 0.7) + 5 - 20, 0);
   statsWindow.headerRow[2]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft); 
   statsWindow.headerRow[2]:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
   statsWindow.headerRow[2]:SetForeColor(Turbine.UI.Color(1,.9,.5));
   statsWindow.headerRow[2]:SetOutlineColor(Turbine.UI.Color(0,0,0));
   statsWindow.headerRow[2]:SetFontStyle(Turbine.UI.FontStyle.Outline);
	statsWindow.headerRow[2]:SetMultiline(false);
	if ((currentSort == "Character") and (not sortByKillTime)) then
		if (currentSortOrder == "ASC") then
			statsWindow.headerRow[2]:SetText(L.CharacterName.." "..AscSortChar);
		else
			statsWindow.headerRow[2]:SetText(L.CharacterName.." "..DescSortChar);
		end
	else
		statsWindow.headerRow[2]:SetText(L.CharacterName);
	end
	statsWindow.headerRow[2].MouseClick = function(sender, args)
		if (not noDataDisplayed) then
			if (currentSort == "Character") then
				if (((currentSortOrder == "ASC") and (not sortByKillTime))
						or ((currentSortOrder == "DESC") and (sortByKillTime))) then
					_G.currentSortOrder = "DESC";
					statsWindow.headerRow[2]:SetText(L.CharacterName.." "..DescSortChar);
				else
					_G.currentSortOrder = "ASC";
					statsWindow.headerRow[2]:SetText(L.CharacterName.." "..AscSortChar);
				end
			else
				_G.currentSort = "Character";
				_G.currentSortOrder = "DESC"; 
	
				statsWindow.headerRow[1]:SetText(L.MobName);
				statsWindow.headerRow[2]:SetText(L.CharacterName.." "..DescSortChar);
				statsWindow.headerRow[3]:SetText(L.LevelRange);
				statsWindow.headerRow[4]:SetText(L.KillCount);
			end
					
	      RefreshKillsDisplay();
			options:SetCurrentSortText();
		end

		_G.sortByKillTime = false;
		statsWindow.killTimeToggle:SetChecked(false);
	end
   -- kill level range header
   statsWindow.headerRow[3] = Turbine.UI.Label();
   statsWindow.headerRow[3]:SetParent(tmpRow);
   statsWindow.headerRow[3]:SetSize(80, 20);
   statsWindow.headerRow[3]:SetPosition((tblWidth - 80 - 100 - 5), 0);
   statsWindow.headerRow[3]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter); 
   statsWindow.headerRow[3]:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
   statsWindow.headerRow[3]:SetForeColor(Turbine.UI.Color(1,.9,.5));
   statsWindow.headerRow[3]:SetOutlineColor(Turbine.UI.Color(0,0,0));
   statsWindow.headerRow[3]:SetFontStyle(Turbine.UI.FontStyle.Outline);
	statsWindow.headerRow[3]:SetMultiline(false);
	if ((currentSort == "Level") and (not sortByKillTime)) then
		if (currentSortOrder == "ASC") then
			statsWindow.headerRow[3]:SetText(L.LevelRange.." "..AscSortChar);
		else
			statsWindow.headerRow[3]:SetText(L.LevelRange.." "..DescSortChar);
		end
	else
		statsWindow.headerRow[3]:SetText(L.LevelRange);
	end
	statsWindow.headerRow[3].MouseClick = function(sender, args)
		if (not noDataDisplayed) then
			if (currentSort == "Level") then
				if (((currentSortOrder == "ASC") and (not sortByKillTime))
						or ((currentSortOrder == "DESC") and (sortByKillTime))) then
					_G.currentSortOrder = "DESC";
					statsWindow.headerRow[3]:SetText(L.LevelRange.." "..DescSortChar);
				else
					_G.currentSortOrder = "ASC";
					statsWindow.headerRow[3]:SetText(L.LevelRange.." "..AscSortChar);
				end
			else
				_G.currentSort = "Level";
				_G.currentSortOrder = "DESC"; 
	
				statsWindow.headerRow[1]:SetText(L.MobName);
				statsWindow.headerRow[2]:SetText(L.CharacterName);
				statsWindow.headerRow[3]:SetText(L.LevelRange.." "..DescSortChar);
				statsWindow.headerRow[4]:SetText(L.KillCount);
			end
					
	      RefreshKillsDisplay();
			options:SetCurrentSortText();
		end

		_G.sortByKillTime = false;
		statsWindow.killTimeToggle:SetChecked(false);
	end
   -- kill count header
   statsWindow.headerRow[4] = Turbine.UI.Label();
   statsWindow.headerRow[4]:SetParent(tmpRow);
   statsWindow.headerRow[4]:SetSize(100, 20);
   statsWindow.headerRow[4]:SetPosition((tblWidth - 100 - 5), 0);
   statsWindow.headerRow[4]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight); 
   statsWindow.headerRow[4]:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
   statsWindow.headerRow[4]:SetForeColor(Turbine.UI.Color(1,.9,.5));
   statsWindow.headerRow[4]:SetOutlineColor(Turbine.UI.Color(0,0,0));
   statsWindow.headerRow[4]:SetFontStyle(Turbine.UI.FontStyle.Outline);
	statsWindow.headerRow[4]:SetMultiline(false);
	if ((currentSort == "Kills") and (not sortByKillTime)) then
		if (currentSortOrder == "ASC") then
			statsWindow.headerRow[4]:SetText(L.KillCount.." "..AscSortChar);
		else
			statsWindow.headerRow[4]:SetText(L.KillCount.." "..DescSortChar);
		end
	else
		statsWindow.headerRow[4]:SetText(L.KillCount);
	end
	statsWindow.headerRow[4].MouseClick = function(sender, args)
		if (not noDataDisplayed) then
			if (currentSort == "Kills") then
				if (((currentSortOrder == "ASC") and (not sortByKillTime))
						or ((currentSortOrder == "DESC") and (sortByKillTime))) then
					_G.currentSortOrder = "DESC";
					statsWindow.headerRow[4]:SetText(L.KillCount.." "..DescSortChar);
				else
					_G.currentSortOrder = "ASC";
					statsWindow.headerRow[4]:SetText(L.KillCount.." "..AscSortChar);
				end
			else
				_G.currentSort = "Kills";
				_G.currentSortOrder = "DESC"; 
	
				statsWindow.headerRow[1]:SetText(L.MobName);
				statsWindow.headerRow[2]:SetText(L.CharacterName);
				statsWindow.headerRow[3]:SetText(L.LevelRange);
				statsWindow.headerRow[4]:SetText(L.KillCount.." "..DescSortChar);
			end
					
	      RefreshKillsDisplay();
			options:SetCurrentSortText();
		end

		_G.sortByKillTime = false;
		statsWindow.killTimeToggle:SetChecked(false);
	end

	-- message to display when there is no data - create hidden
   local tmpRow = Turbine.UI.Control();
   tmpRow:SetParent(statsWindow.statsTable);
   tmpRow:SetSize(statsWindow.statsTable:GetWidth(), 0);
	statsWindow.noRowsMessage = Turbine.UI.Label();
	statsWindow.noRowsMessage:SetParent(tmpRow);
	statsWindow.noRowsMessage:SetSize(tmpRow:GetWidth() - 10, 20);
	statsWindow.noRowsMessage:SetPosition(5, 0);
	statsWindow.noRowsMessage:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft); 
	statsWindow.noRowsMessage:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	statsWindow.noRowsMessage:SetMultiline(false);
	statsWindow.noRowsMessage:SetText(L.NoRowsCurrentChar);
	statsWindow.statsTable:AddItem(tmpRow);
   
   -- define a function to save new window size/position settings when moved/resized
   statsWindow.SaveState = function()
      -- only save the settings if the window is visible
      if (statsWindow:IsVisible()) then
         settings.leftPos, settings.topPos = statsWindow:GetPosition();
         settings.width, settings.height = statsWindow:GetSize();
   
         -- save settings to Character-level file
         SaveSettings();
      end
   end

   -- tables to hold individual kill log UI element rows and separator lines
   statsWindow.rowSeparators = {};
   statsWindow.killRows = {};

   -- refresh layout of window after resize
   statsWindow.content.Layout = function()
      statsWindow.statsTable:SetSize(statsWindow.content:GetWidth() - 35, statsWindow.content:GetHeight() - 75);

      local newWidth = statsWindow.statsTable:GetWidth();

      statsWindow.vscroll:SetLeft(statsWindow.statsTable:GetLeft() + newWidth);
      statsWindow.vscroll:SetHeight(statsWindow.statsTable:GetHeight());

      statsWindow.allCharactersToggle:SetTop(statsWindow:GetHeight() - 45);

		statsWindow.killTimeToggle:SetWidth((statsWindow:GetWidth() / 2) - 40);
		statsWindow.killTimeToggle:SetPosition((statsWindow:GetWidth() / 2), statsWindow:GetHeight() - 45);

		statsWindow.dataFailure:SetWidth(statsWindow.content:GetWidth() - 80);

	   statsWindow.optionsLink:SetLeft(statsWindow.content:GetWidth() - 115);

      local tmpRow = statsWindow.noRowsMessage:GetParent();
      tmpRow:SetWidth(newWidth);

		statsWindow.noRowsMessage:SetWidth(newWidth - 10);

      local tmpRow = statsWindow.headerRow[1]:GetParent();
      tmpRow:SetWidth(newWidth);

      if (showAllCharacters) then
         statsWindow.headerRow[1]:SetWidth(((newWidth - 80 - 100 - 10) * 0.7) - 5 - 20);
         statsWindow.headerRow[2]:SetWidth(((newWidth - 80 - 100 - 10) * 0.3) - 5 + 20);
         statsWindow.headerRow[2]:SetLeft(((newWidth - 80 - 100 - 10) * 0.7) + 5 - 20);
      else
         statsWindow.headerRow[1]:SetWidth(newWidth - 80 - 100 - 10);
         statsWindow.headerRow[2]:SetWidth(0);
      end

      statsWindow.headerRow[3]:SetLeft(newWidth - 80 - 100 - 5);
      statsWindow.headerRow[4]:SetLeft(newWidth - 100 - 5);
   
      for i = 1, #statsWindow.killRows do
         local tmpRow = statsWindow.killRows[i][1]:GetParent();
         tmpRow:SetWidth(newWidth);

         statsWindow.rowSeparators[i]:SetSize(newWidth, 1);
         
         if (showAllCharacters) then
            statsWindow.killRows[i][1]:SetWidth(((newWidth - 80 - 100 - 10) * 0.7) - 5 - 20);
            statsWindow.killRows[i][2]:SetWidth(((newWidth - 80 - 100 - 10) * 0.3) - 5 + 20);
            statsWindow.killRows[i][2]:SetLeft(((newWidth - 80 - 100 - 10) * 0.7) + 5 - 20);
         else
            statsWindow.killRows[i][1]:SetWidth(newWidth - 80 - 100 - 10);
            statsWindow.killRows[i][2]:SetWidth(0);
         end

         statsWindow.killRows[i][3]:SetLeft(newWidth - 80 - 100 - 5);
         statsWindow.killRows[i][4]:SetLeft(newWidth - 100 - 5);
      end
   end

	-- create icon to quickly open main KillTracker UI window
	_G.minWindow = MinimizedWindow();
	minWindow:SetPosition(settings.iconLeft, settings.iconTop);
   
   -- define a function to save new icon position settings when moved
   minWindow.SaveState = function()
      -- only save the settings if the window is visible
      if (minWindow:IsVisible()) then
         settings.iconLeft, settings.iconTop = minWindow:GetPosition();
   
         -- save settings to Character-level file
         SaveSettings();
      end
   end
end

function sortByEnemyDESC(arg1, arg2)
	if (arg1[1] < arg2[1]) then
		return(true);
	elseif ((arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

function sortByEnemyASC(arg1, arg2)
	if (arg1[1] > arg2[1]) then
		return(true);
	elseif ((arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

function sortByCharacterDESC(arg1, arg2)
	if (arg1[5] < arg2[5]) then
		return(true);
	elseif ((arg1[5] == arg2[5]) and (arg1[1] < arg2[1])) then
		return(true);
	end
end

function sortByCharacterASC(arg1, arg2)
	if (arg1[5] > arg2[5]) then
		return(true);
	elseif ((arg1[5] == arg2[5]) and (arg1[1] < arg2[1])) then
		return(true);
	end
end

function sortByLevelDESC(arg1, arg2)
	if (arg1[3] < arg2[3]) then
		return(true);
	elseif ((arg1[3] == arg2[3]) and (arg1[1] < arg2[1])) then
		return(true);
	elseif ((arg1[3] == arg2[3]) and (arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

function sortByLevelASC(arg1, arg2)
	if (arg1[3] > arg2[3]) then
		return(true);
	elseif ((arg1[3] == arg2[3]) and (arg1[1] < arg2[1])) then
		return(true);
	elseif ((arg1[3] == arg2[3]) and (arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

function sortByKillsDESC(arg1, arg2)
	if (arg1[2] < arg2[2]) then
		return(true);
	elseif ((arg1[2] == arg2[2]) and (arg1[1] < arg2[1])) then
		return(true);
	elseif ((arg1[2] == arg2[2]) and (arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

function sortByKillsASC(arg1, arg2)
	if (arg1[2] > arg2[2]) then
		return(true);
	elseif ((arg1[2] == arg2[2]) and (arg1[1] < arg2[1])) then
		return(true);
	elseif ((arg1[2] == arg2[2]) and (arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

function sortByKillTimeDESC(arg1, arg2)
	if (arg1[6] > arg2[6]) then
		return(true);
	elseif ((arg1[6] == arg2[6]) and (arg1[1] < arg2[1])) then
		return(true);
	elseif ((arg1[6] == arg2[6]) and (arg1[1] == arg2[1]) and (arg1[5] < arg2[5])) then
		return(true);
	end
end

-- refresh the results table in the window 
function _G.RefreshKillsDisplay()
   local rowsDisplayed = 0;

   -- only do this if the window is visible and there are results to display
   if (statsWindow:IsVisible() and (#killsTable > 0)) then
		local sortedKills = {};

		for i = 1, #killsTable do
			sortedKills[i] = killsTable[i];
		end
   
	   -- sort the table alpha by target name if there is more than one record
	   if (#sortedKills > 1) then
	   	if (sortByKillTime) then
	   		table.sort(sortedKills, sortByKillTimeDESC);
	   	elseif (currentSort == "Character") then
	   		if (currentSortOrder == "ASC") then
	      		table.sort(sortedKills, sortByCharacterASC);
	      	else
	      		table.sort(sortedKills, sortByCharacterDESC);
	      	end
	      elseif (currentSort == "Level") then
	   		if (currentSortOrder == "ASC") then
	      		table.sort(sortedKills, sortByLevelASC);
	      	else
	      		table.sort(sortedKills, sortByLevelDESC);
	      	end
	      elseif (currentSort == "Kills") then
	   		if (currentSortOrder == "ASC") then
	      		table.sort(sortedKills, sortByKillsASC);
	      	else
	      		table.sort(sortedKills, sortByKillsDESC);
	      	end
	      else -- currentSort == "Enemy"
	   		if (currentSortOrder == "ASC") then
	      		table.sort(sortedKills, sortByEnemyASC);
	      	else
	      		table.sort(sortedKills, sortByEnemyDESC);
	      	end
			end
	   end
	
	   local tblWidth = statsWindow.statsTable:GetWidth();
		local enemyName = "";
		local characterName = "";
	
	   -- loop through master kills log
	   for i = 1, #sortedKills do
	      -- only display kills for this character unless showAllCharacters is set
	      if ((sortedKills[i][5] == player.uniqueName) or (showAllCharacters)) then
	         -- increment row count for positioning
	         rowsDisplayed = rowsDisplayed + 1;

	         -- update existing UI element row if it exists
	         if (#statsWindow.killRows >= rowsDisplayed) then
         		local tmpRow = statsWindow.killRows[rowsDisplayed][1]:GetParent();
         		tmpRow:SetHeight(22);
	         -- add a new row
	         else
	            statsWindow.killRows[rowsDisplayed] = {};
	         
	            local tmpRow = Turbine.UI.Control();
	            tmpRow:SetParent(statsWindow.statsTable);
	            tmpRow:SetSize(tblWidth, 22);
	            tmpRow:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	            -- even row highlighting
	            if (rowsDisplayed % 2 == 0) then
	               tmpRow:SetBackColor(Turbine.UI.Color(0.1, 1, 1, 1));
	            end

					-- row separator
					statsWindow.rowSeparators[rowsDisplayed] = Turbine.UI.Control();
					statsWindow.rowSeparators[rowsDisplayed]:SetParent(tmpRow);
	            statsWindow.rowSeparators[rowsDisplayed]:SetSize(tblWidth, 1);
	            statsWindow.rowSeparators[rowsDisplayed]:SetPosition(0, 0);
	            statsWindow.rowSeparators[rowsDisplayed]:SetBackColor(Turbine.UI.Color(.3, .3, .1));
	            -- target name
	            statsWindow.killRows[rowsDisplayed][1] = Turbine.UI.Label();
	            statsWindow.killRows[rowsDisplayed][1]:SetParent(tmpRow);
	            if (showAllCharacters) then
	               statsWindow.killRows[rowsDisplayed][1]:SetSize(((tblWidth - 80 - 100 - 10) * 0.7) - 5 - 20, 20);
	            else
	               statsWindow.killRows[rowsDisplayed][1]:SetSize((tblWidth - 80 - 100 - 10), 20);
	            end
	            statsWindow.killRows[rowsDisplayed][1]:SetPosition(5, 1);
	            statsWindow.killRows[rowsDisplayed][1]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft); 
	            statsWindow.killRows[rowsDisplayed][1]:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	            statsWindow.killRows[rowsDisplayed][1]:SetFont(Turbine.UI.Lotro.Font.Verdana12);
					statsWindow.killRows[rowsDisplayed][1]:SetMultiline(false);
	            -- character name(server ID)
	            statsWindow.killRows[rowsDisplayed][2] = Turbine.UI.Label();
	            statsWindow.killRows[rowsDisplayed][2]:SetParent(tmpRow);
	            statsWindow.killRows[rowsDisplayed][2]:SetSize(((tblWidth - 80 - 100 - 10) * 0.3) - 5 + 20, 20);
	            statsWindow.killRows[rowsDisplayed][2]:SetPosition(((tblWidth - 80 - 100 - 10) * 0.7) + 5 - 20, 1);
	            statsWindow.killRows[rowsDisplayed][2]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft); 
	            statsWindow.killRows[rowsDisplayed][2]:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	            statsWindow.killRows[rowsDisplayed][2]:SetFont(Turbine.UI.Lotro.Font.Verdana12);
					statsWindow.killRows[rowsDisplayed][2]:SetMultiline(false);
	            if (not showAllCharacters) then
	               statsWindow.killRows[rowsDisplayed][2]:SetSize(0, 20);
	            end
	            -- kill level range
	            statsWindow.killRows[rowsDisplayed][3] = Turbine.UI.Label();
	            statsWindow.killRows[rowsDisplayed][3]:SetParent(tmpRow);
	            statsWindow.killRows[rowsDisplayed][3]:SetSize(80, 20);
	            statsWindow.killRows[rowsDisplayed][3]:SetPosition((tblWidth - 80 - 100 - 5), 1);
	            statsWindow.killRows[rowsDisplayed][3]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter); 
	            statsWindow.killRows[rowsDisplayed][3]:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	            statsWindow.killRows[rowsDisplayed][3]:SetFont(Turbine.UI.Lotro.Font.Verdana12);
					statsWindow.killRows[rowsDisplayed][3]:SetMultiline(false);
	            -- kill count
	            statsWindow.killRows[rowsDisplayed][4] = Turbine.UI.Label();
	            statsWindow.killRows[rowsDisplayed][4]:SetParent(tmpRow);
	            statsWindow.killRows[rowsDisplayed][4]:SetSize(95, 20);
	            statsWindow.killRows[rowsDisplayed][4]:SetPosition((tblWidth - 100 - 5), 1);
	            statsWindow.killRows[rowsDisplayed][4]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight); 
	            statsWindow.killRows[rowsDisplayed][4]:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	            statsWindow.killRows[rowsDisplayed][4]:SetFont(Turbine.UI.Lotro.Font.Verdana12);
					statsWindow.killRows[rowsDisplayed][4]:SetMultiline(false);
	         
	            statsWindow.statsTable:AddItem(tmpRow);
	         end
	         
				-- if sorting by enemy:
				--    show the top row border if the current line has a different value as the last line
				-- 	hide the text if the current line has the same value as the last line  
				if (currentSort == "Enemy") then
					if (sortedKills[i][1] ~= enemyName) then
            		statsWindow.rowSeparators[rowsDisplayed]:SetVisible(true);
            		statsWindow.killRows[rowsDisplayed][1]:SetText(sortedKills[i][1]);
            	else
            		statsWindow.rowSeparators[rowsDisplayed]:SetVisible(false);
            		statsWindow.killRows[rowsDisplayed][1]:SetText("");
            	end
 				-- if sorting by character:
				--    show the top row border if the current line has a different value as the last line
				elseif (currentSort == "Character") then
          		statsWindow.killRows[rowsDisplayed][1]:SetText(sortedKills[i][1]);

					if (sortedKills[i][5] ~= characterName) then
            		statsWindow.rowSeparators[rowsDisplayed]:SetVisible(true);
            	else
            		statsWindow.rowSeparators[rowsDisplayed]:SetVisible(false);
            	end
				else
            	statsWindow.rowSeparators[rowsDisplayed]:SetVisible(false);
            	statsWindow.killRows[rowsDisplayed][1]:SetText(sortedKills[i][1]);
            end        

           	statsWindow.killRows[rowsDisplayed][2]:SetText(characterNames[sortedKills[i][5]]);

				-- if first and most recent level are the same, just display it once
            if (sortedKills[i][3] == sortedKills[i][4]) then
            	statsWindow.killRows[rowsDisplayed][3]:SetText(FormatNumber(sortedKills[i][3]));
            else
            	statsWindow.killRows[rowsDisplayed][3]:SetText(FormatNumber(sortedKills[i][3]).."-"..FormatNumber(sortedKills[i][4]));
            end

				if ((newKills[sortedKills[i][1]] == nil) or (sortedKills[i][5] ~= player.uniqueName)) then
					statsWindow.killRows[rowsDisplayed][4]:SetText(FormatNumber(sortedKills[i][2]));
				else
					statsWindow.killRows[rowsDisplayed][4]:SetText(FormatNumber(sortedKills[i][2]).."("..FormatNumber(newKills[sortedKills[i][1]][1])..")");
				end
	      
		      enemyName = sortedKills[i][1];
		      characterName = sortedKills[i][5];
	      end
	   end
	end

	-- hide rows that no longer have data, setting height to 0 for the scrollbar
	if (#statsWindow.killRows > rowsDisplayed) then 
		for i = rowsDisplayed + 1, #statsWindow.killRows do
         local tmpRow = statsWindow.killRows[i][1]:GetParent();
         tmpRow:SetHeight(0);
		end
	end

   -- hide/show the No Data message, setting height to 0 for the scrollbar
	local tmpRow = statsWindow.noRowsMessage:GetParent();

   if (rowsDisplayed == 0) then
		_G.noDataDisplayed = true;
      tmpRow:SetHeight(20);
	else
		_G.noDataDisplayed = false;
		tmpRow:SetHeight(0);
   end
end
